package com.gradecak.alfresco.mvc;

import javax.servlet.ServletContext;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.extensions.webscripts.AbstractWebScript;
import org.springframework.extensions.webscripts.WebScriptRequest;
import org.springframework.extensions.webscripts.WebScriptResponse;
import org.springframework.extensions.webscripts.servlet.WebScriptServletRequest;
import org.springframework.extensions.webscripts.servlet.WebScriptServletResponse;
import org.springframework.util.Assert;
import org.springframework.web.context.ServletContextAware;
import org.springframework.web.context.WebApplicationContext;
import org.springframework.web.servlet.DispatcherServlet;

public class DispatcherWebScript extends AbstractWebScript implements InitializingBean, ServletContextAware,
		ApplicationContextAware {

	private static final Log logger = LogFactory.getLog(DispatcherWebScript.class);
	private DispatcherServlet s;
	private ServletContext servletContext;
	private String contextConfigLocation;
	private ApplicationContext applicationContext;

	public void execute(final WebScriptRequest req, final WebScriptResponse res) {
		FrameworkServletRequestProxy proxy = new FrameworkServletRequestProxy((WebScriptServletRequest) req);

		try {
			s.service(proxy, ((WebScriptServletResponse) res).getHttpServletResponse());
		} catch (Exception e) {
			String msg = "Could not process with the request for " + proxy.getRequestURI();
			logger.error(msg, e);
			throw new FrameworkRuntimeException(msg, e);
		}
	}

	public void afterPropertiesSet() throws Exception {
		Assert.notNull(contextConfigLocation, "contextConfigLocation must be initialized");
		Assert.notNull(servletContext, "servletContext must be initialized");

		s = new FrameworkDispatcherServlet((WebApplicationContext) this.applicationContext);
		s.setContextConfigLocation(contextConfigLocation);
		s.init(new FrameworkServletConfig(servletContext));

		logger.info("DispatcherWebScript initialized");
	}

	public void setContextConfigLocation(String contextConfigLocation) {
		this.contextConfigLocation = contextConfigLocation;
	}

	public void setServletContext(ServletContext servletContext) {
		this.servletContext = servletContext;
	}

	public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
		this.applicationContext = applicationContext;
	}
}
