package com.gradecak.alfresco.mvc;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;

import org.alfresco.service.ServiceRegistry;
import org.springframework.beans.BeansException;
import org.springframework.context.event.ContextRefreshedEvent;
import org.springframework.web.context.WebApplicationContext;
import org.springframework.web.servlet.DispatcherServlet;

public class FrameworkDispatcherServlet extends DispatcherServlet {

	/**
	 * 
	 */
	private static final long serialVersionUID = 7192691111191267359L;

	/** Flag used to detect whether onRefresh has already been called */
	private boolean refreshEventReceived = false;

	private ServiceRegistry serviceRegistry;
	private WebApplicationContext parentWebApplicationContext;

	public FrameworkDispatcherServlet(WebApplicationContext webApplicationContext) {
		this.parentWebApplicationContext = webApplicationContext;
	}

	@Override
	protected WebApplicationContext initWebApplicationContext() throws BeansException {

		WebApplicationContext wac = createWebApplicationContext(parentWebApplicationContext);

		if (!this.refreshEventReceived) {
			// Apparently not a ConfigurableApplicationContext with refresh
			// support:
			// triggering initial onRefresh manually here.
			onRefresh(wac);
		}

		if (wac == null) {
			wac = super.initWebApplicationContext();
		}

		return wac;

	}

	public void onApplicationEvent(ContextRefreshedEvent event) {
		this.refreshEventReceived = true;
		onRefresh(event.getApplicationContext());
	}

	@Override
	public void service(final ServletRequest req, final ServletResponse res) throws ServletException, IOException {
		doService(req, res);
	}

	protected void doService(final ServletRequest req, final ServletResponse res) throws ServletException, IOException {
		super.service(req, res);
	}
}
